from typing import Final

KERNEL_CANCELLED: Final[str] = "kernel_cancelled"
KERNEL_CREATING: Final[str] = "kernel_creating"
KERNEL_STARTED: Final[str] = "kernel_started"
KERNEL_TERMINATED: Final[str] = "kernel_terminated"
KERNEL_TERMINATING: Final[str] = "kernel_terminating"
SESSION_FAILURE: Final[str] = "session_failure"
SESSION_STARTED: Final[str] = "session_started"
SESSION_SUCCESS: Final[str] = "session_success"
SESSION_TERMINATED: Final[str] = "session_terminated"
